import React, { useState, useEffect } from 'react';
import ColorPicker from '@graphpolaris/shared/lib/components/colorComponents/colorPicker';
import { VisualizationSettingsPropTypes } from '@graphpolaris/shared/lib/vis/common';
import { MapProps } from '../../mapvis';
import { Button, EntityPill, Icon, Input, RelationPill } from '@graphpolaris/shared/lib/components';
import { SubdirectoryArrowRight, Visibility, VisibilityOff } from '@mui/icons-material';
import { IconButton } from '@graphpolaris/shared/lib/components/buttons/button.stories';

export default function NodeLinkOptions({ settings, graphMetadata, updateSettings }: VisualizationSettingsPropTypes<MapProps>) {
  const [collapsed, setCollapsed] = useState<Record<string, boolean>>({});

  useEffect(() => {
    graphMetadata.nodes.labels.forEach((node) => {
      updateSettings({
        [node]: {
          color: [0, 0, 0],
          hidden: false,
          shape: '',
          size: 10,
          fixed: true,
          min: 0,
          max: 10,
          sizeAttribute: '',
          lon: '',
          lat: '',
          ...settings?.[node],
        },
      });
    });

    graphMetadata.edges.labels.forEach((edge) => {
      updateSettings({
        [edge]: {
          color: [0, 0, 0],
          hidden: false,
          fixed: true,
          min: 0,
          max: 10,
          width: 1,
          widthAttribute: '',
          ...settings?.[edge],
        },
      });
    });
  }, [graphMetadata]);

  const spatialAttributes: { [id: string]: string[] } = {};
  graphMetadata.nodes.labels.forEach((node) => {
    spatialAttributes[node] = Object.entries(graphMetadata.nodes.types[node].attributes)
      .filter(([, value]) => value.dimension === 'numerical')
      .map(([key]) => key);
  });

  const handleCollapseToggle = (nodeType: string) => {
    setCollapsed((prevCollapsed) => ({
      ...prevCollapsed,
      [nodeType]: !prevCollapsed[nodeType],
    }));
  };

  return (
    <div>
      {graphMetadata.nodes.labels.map((nodeType) => (
        <div className="mt-2" key={nodeType}>
          <div className="flex items-center">
            <div className="flex flex-grow mr-2 cursor-pointer" onClick={() => handleCollapseToggle(nodeType)}>
              <EntityPill title={nodeType} />
            </div>
            <div className="flex items-center space-x-2">
              <ColorPicker
                value={settings?.[nodeType]?.['color'] ? settings?.[nodeType]?.['color'] : [0, 0, 0]}
                updateValue={(val: number[]) => updateSettings({ [nodeType]: { ...settings?.[nodeType], color: val } })}
              />
              <Button
                iconComponent={settings?.[nodeType]?.hidden ? <VisibilityOff /> : <Visibility />}
                variant="ghost"
                onClick={() => {
                  updateSettings({ [nodeType]: { ...settings?.[nodeType], hidden: !settings?.[nodeType]?.hidden as boolean } });
                }}
              />
            </div>
          </div>

          {!collapsed[nodeType] && (
            <div>
              <Input
                inline
                label="Longitude"
                type="dropdown"
                value={settings?.[nodeType]?.lon}
                options={[...spatialAttributes[nodeType]]}
                disabled={!settings.node || spatialAttributes[nodeType].length < 1}
                onChange={(val) => updateSettings({ [nodeType]: { ...settings?.[nodeType], lon: val as string } })}
              />

              <Input
                inline
                label="Latitude"
                type="dropdown"
                value={settings?.[nodeType]?.lat}
                options={[...spatialAttributes[nodeType]]}
                disabled={!settings.node || spatialAttributes[nodeType].length < 1}
                onChange={(val) => updateSettings({ [nodeType]: { ...settings?.[nodeType], lat: val as string } })}
              />
              <Input
                inline
                label="Shape"
                type="dropdown"
                value={settings?.[nodeType]?.shape}
                options={['circle', 'square', 'triangle', 'diamond', 'location', 'star']}
                disabled={!settings.shape}
                onChange={(val) => updateSettings({ [nodeType]: { ...settings?.[nodeType], shape: val as string } })}
              />
              <Input
                label="Size"
                type="slider"
                min={0}
                max={20}
                step={1}
                value={settings?.[nodeType]?.size}
                onChange={(val) => updateSettings({ [nodeType]: { ...settings?.[nodeType], size: val as number } })}
              />
            </div>
          )}
        </div>
      ))}

      {graphMetadata.edges.labels.map((edgeType) => (
        <div className="mt-2" key={edgeType}>
          <div className="flex items-center">
            <div className="w-3/4 mr-6 cursor-pointer" onClick={() => handleCollapseToggle(edgeType)}>
              <RelationPill title={edgeType} />
            </div>
            <div className="w-1/2 flex">
              <ColorPicker
                value={settings?.[edgeType]?.['color'] ? settings?.[edgeType]?.['color'] : [0, 0, 0]}
                updateValue={(val: number[]) => updateSettings({ [edgeType]: { ...settings?.[edgeType], color: val } })}
              />
              <Button
                iconComponent={settings?.[edgeType]?.hidden ? <VisibilityOff /> : <Visibility />}
                variant="ghost"
                onClick={() => {
                  updateSettings({ [edgeType]: { ...settings?.[edgeType], hidden: !settings?.[edgeType]?.hidden as boolean } });
                }}
              />
            </div>
          </div>

          {!collapsed[edgeType] && (
            <div>
              <Input
                label="Enable brushing"
                type="boolean"
                value={settings.enableBrushing}
                onChange={(val) => {
                  updateSettings({ enableBrushing: val as boolean });
                }}
              />

              <div>
                <div className="flex items-center gap-1">
                  <Icon component={<SubdirectoryArrowRight />} size={16} color="text-secondary-300" />
                  <span>Width</span>
                </div>
                <Input
                  label="Fixed"
                  type="boolean"
                  value={settings?.[edgeType]?.fixed ?? false}
                  onChange={(val: boolean) => updateSettings({ [edgeType]: { ...settings?.[edgeType], fixed: val } })}
                />
                {!settings?.[edgeType]?.fixed ? (
                  <div>
                    <Input
                      label="Based on"
                      type="dropdown"
                      size="xs"
                      options={
                        graphMetadata.nodes.types[edgeType]?.attributes
                          ? Object.keys(graphMetadata.nodes.types[edgeType].attributes).filter(
                              (key) => graphMetadata.nodes.types[edgeType].attributes[key].dimension === 'numerical',
                            )
                          : []
                      }
                      value={settings?.[edgeType]?.sizeAttribute ?? ''}
                      onChange={(val: string | number) => updateSettings({ [edgeType]: { ...settings?.[edgeType], sizeAttribute: val } })}
                    />
                    <div className="flex">
                      <Input
                        type="number"
                        label="min"
                        size="xs"
                        value={settings?.[edgeType]?.min ?? 0}
                        onChange={(val: number) => updateSettings({ [edgeType]: { ...settings?.[edgeType], min: val } })}
                      />
                      <Input
                        type="number"
                        label="max"
                        size="xs"
                        value={settings?.[edgeType]?.max ?? 10}
                        onChange={(val: number) => updateSettings({ [edgeType]: { ...settings?.[edgeType], max: val } })}
                      />
                    </div>
                  </div>
                ) : (
                  <div>
                    <Input
                      type="slider"
                      label="Width"
                      min={0}
                      max={10}
                      step={0.5}
                      value={settings?.[edgeType]?.width ?? 1}
                      onChange={(val: number) => updateSettings({ [edgeType]: { ...settings?.[edgeType], width: val } })}
                    />
                  </div>
                )}
              </div>
            </div>
          )}
        </div>
      ))}
    </div>
  );
}
